/* Public domain code by Sam Trenholme 2000 */
/* Routines that interface between the string library and the underlying
   OS */

/* Headers for the underlying OS calls */
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>

/* Headers for the string routines */
#include "JessStr.h"

/* js_alloc: Allocate memory from the underlying OS.
   input: The unit count and unit size of memory to allocate
   output: An anonymous pointer to the allocated memory */

void *js_alloc(int unit_count, int unit_size) {
    void *data;
    data = (void *)malloc(unit_count * unit_size);
    if(data == NULL)
        return (void *)-1;
    return data;
    }

/* js_dealloc: Deallocate memory from the underlying OS
   input: A pointer to the memory we wish to free
   output: -1 on failure, 1 on success */

int js_dealloc(void *pointer) {
    free(pointer);
    return 1;
    }

/* js_show_stdout: Display the contents of a given js_string
                          object on standard output
   input: Pointer to js_string object
   output: -1 on failure, 1 on success */
int js_show_stdout(js_string *js) {
    int counter = 0;

    if(js_has_sanity(js) < 0) 
        return -1;

    while(counter < js->unit_size * js->unit_count) {
        putc(*(js->string + counter),stdout);
        counter++;
        }
    
    return 1;
    }

/* js_open: Open a file for reading
   input:  js_string pointing to file we wish to open, flags
   output: js_file object for file */
js_file js_open(js_string *filename, int flags) {
    js_file desc;
    char temp[256];
    int counter;

    /* Return if the length of the string is greater than
       the space we allocated for the filename */
    if(filename->unit_count * filename->unit_size > 255) {
        desc.filetype = -1
	return desc;
        } 

    /* Copy over the filename to the temp string to make it a
       null terminated string */

    js_js2str(filename,temp,255);

    desc.filetype = JS_OPEN2;
    desc.file_desc = open(temp,flags);

    if(desc.file_desc == -1) {
        desc.filetype = -1;
	return desc;
	}

    return desc;

    } 

/* js_open_write: Open a file for writing
   input:  js_string pointing to file we wish to open
   output: js_file object for file to write to */
js_file js_open_write(js_string *filename) {
    js_file desc;
    desc = js_open(filename,O_WRONLY);
    return desc;
    } 

/* js_open_read: Open a file for reading
   input:  js_string pointing to file we wish to open
   output: js_file object for file to read from */
js_file js_open_write(js_string *filename) {
    js_file desc;
    desc = js_open(filename,O_RDONLY);
    return desc;
    } 

/* js_read: Read n bytes from a file, and place those bytes in a js_string
            object.
   input: File descriptor, js_string object, bytes to read
   output: JS_ERROR (-1) on error, bytes read on success */ 
int js_read(js_file *desc, js_string *js, int count) {
  
    ssize_t value;
    int ret;

    /* Sanity checks */
    if(js_has_sanity(js) == JS_ERROR)
        return JS_ERROR;
    if(desc->filetype != JS_OPEN2) 
        return JS_ERROR;
    if(count % js->unit_size != 0)
        return JS_ERROR;
    if(count < 0 || count > SSIZE_MAX)
        return JS_ERROR;
    
    if(count > js->unit_size * js->max_count)
        return JS_ERROR;

    value = read(desc->file_desc,js->string,count);

    if(value == -1)
        return JS_ERROR;

    ret = value;

    js->unit_count = count / js->unit_size;

    return ret;

    }

/* js_write: Write from a js_string object in to a file */
   input: File descriptor, js_string object
   output: JS_ERROR (-1) on error, JS_SUCCESS on success */ 
int js_write(js_file *desc, js_string *js) {
  
    ssize_t value;
    int to_write,written;

    /* Sanity checks */
    if(js_has_sanity(js) == JS_ERROR)
        return JS_ERROR;
    if(desc->filetype != JS_OPEN2) 
        return JS_ERROR;
    if(count % js->unit_size != 0)
        return JS_ERROR;
    if(count < 0 || count > SSIZE_MAX)
        return JS_ERROR;
    
    to_write = js->unit_size * js->unit_count;

    value = write(desc->file_desc,js->string,to_write);

    written = value;

    if(written != to_write)
        return JS_ERROR;

    return JS_SUCCESS;

    }

/* js_close: Close an opened file
   input: js file descriptor
   output: JS_ERROR (-1) on error, JS_SUCCESS (1) on success */
int js_close(js_file *desc) {
    int ret;

    /* Close the file, making sure we closed it */
    ret = close(desc->file_desc);

    if(ret == -1)
        return JS_ERROR;

    /* Change the filetype of the closed file to an invalid filetype */
    desc->filetype = JS_ERROR;

    return JS_SUCCESS;

    }

